/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Full Fullscreen Firefox extension.
 *
 * The Initial Developer of the Original Code is
 * emsearcy@osuosl.org
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  DISRUPTIVE INNOVATIONS SARL
 *  Francek Prijatelj
 *
 *
 * ***** END LICENSE BLOCK ***** */

var FS_data = {

  // booleans reflecting full and fuller screen
  mIsFullScreen: false,
  mIsFullerScreen: false,

  // true if we're in fuller screen mode and the context makes us
  // fallback to normal fullscreen mode
  mForceStdFullScreen: false,

  // we'll have to hide and show again tabs
  mTabs: null,

  // and we need the screen height for bottom edge collision
  mScreenHeight: 0,

  // we want to show the statusbar when the status changes
  mStatusbarDisplay: null,
  mStatusbarDisplayText: "",
  mStatusbarDisplayTimeout: null,

  // the status bar icon
  mStatusbarIcon: null,

  // the find toolbar
  mFindToolbar: null,

  mSlideShowMenupopup: null,

  mPrefs: null,

  // threshold for detection of edge collision
  kThreshold: 10,

  // URL constants
  kBROWSER_CHROME_URI: "chrome://browser/content/browser.xul",
  kXUL_NS: "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",
  kIOServiceCID: "@mozilla.org/network/io-service;1",
  kSTRINGBUNDLE: "chrome://fullerscreen/locale/fullerscreen.properties",

  kTHRESHOLD_PREF: "extensions.fullerscreen.edgeCollisionThreshold",
  kTIMEOUT_PREF: "extensions.fullerscreen.statusbarTimeout",
  kBAD_MIMETYPES_PREF: "extensions.fullerscreen.incompatibleMIMETypes",
  kHYPERLINKS_PREF: "extensions.fullerscreen.statusbar.showWhenMessageChanges",
  kSHOW_ICON_PREF: "extensions.fullerscreen.statusbar.showFSIcon",
  kTHUMBNAIL_HEIGHT_PREF: "extensions.fullerscreen.thumbnails.height"
}

var FS_urlLocationChange =
{
  onStateChange:function(aWebProgress,
                         aRequest,
                         aStateFlags,
                         aStatus)
  { /* STUB */ },

  onProgressChange:function(aWebProgress,
                            aRequest,
                            aCurSelfProgress,
                            aMaxSelfProgress,
                            aCurTotalProgress,
                            aMaxTotalProgress)
  { /* STUB */ },

  onStatusChange:function(aWebProgress,
                          aRequest,
                          aStatus,
                          aMessage)
  { /* STUB */ },

  onSecurityChange:function(aWebProgress,
                            aRequest,
                            aState)
  { /* STUB */ },

  onLocationChange:function(aWebProgress, aRequest, aLocation)
  {
    // sanity check, do we have a URL here ?
    if (aLocation)
      CheckIfFullScreen()
  }
}

function CheckIfFullScreen()
{
  // we care only  if we're in full screen mode
  if (window.fullScreen)
  {
    CheckIfProjection();
    // so what's the MIME type of the content the browser just loaded ?
    var contentType = window.getBrowser().contentDocument.contentType;

    // is it one of the types that don't fit well with fullerscreen ?
    if (MIMETypeCantCopeWithFullerScreen(contentType))
    {
      // force normal full screen mode
      FS_data.mForceStdFullScreen = true;

      // unhide the toolbars, statusbar and tabs.
      FS_toggleElementsForFullerScreen("toolbar", true, true);
      FS_toggleElementsForFullerScreen("statusbar", true, true);
      FS_data.mTabs.removeAttribute("moz-collapsed");
      FS_data.mIsFullerScreen = true;
    }
    else
    {
      // nah, this MIME type is safe...
      FS_data.mForceStdFullScreen = false;

      // people who live hidden live well...
      FS_data.mIsFullerScreen = false;
      FS_toggleElementsForFullerScreen("toolbar", true, false);
      FS_toggleElementsForFullerScreen("statusbar", true, false);
      FS_data.mTabs.setAttribute("moz-collapsed", "true");
    }
  }
  else
    FS_data.mForceStdFullScreen = false;
}

var FS_prefObserver = {
  _self: this,

  QueryInterface: function(aIID) {
    if (aIID.equals(Components.interfaces.nsIObserver) ||
        aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
        aIID.equals(Components.interfaces.nsISupports))
      return this;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  observe: function(aSubject, aTopic, aPrefName) {
    if (aTopic != "nsPref:changed")
      return;

    var prefsvc =
      aSubject.QueryInterface(Components.interfaces.nsIPrefBranch2);

    switch (aPrefName) {
      case FS_data.kSHOW_ICON_PREF:
        var showIcon = prefsvc.getBoolPref(aPrefName);
        if (showIcon)
          FS_data.mStatusbarIcon.removeAttribute("hidden");
        else
          FS_data.mStatusbarIcon.setAttribute("hidden", "true");
        break;
    }
  }
}

function ShowFindToolbar(aEvent)
{
  if (FS_data.mFindToolbar) {
    FS_data.mFindToolbar.removeAttribute("hidden");
    FS_data.mFindToolbar.removeAttribute("moz-collapsed");
  }
}

function fullerScreenInit()
{
  // store a few XUL elements so we don't have to call getElementById()
  // again and again
  FS_data.mTabs = document.getElementById("content").mStrip;
  FS_data.mStatusbarDisplay = document.getElementById("statusbar-display");
  FS_data.mScreenHeight = screen.height;
  FS_data.mStatusbarIcon = document.getElementById("full-screen-statusbar-item");
  FS_data.mFindToolbar = document.getElementById("FindToolbar");

  // we want to be notified when fullscreen mode is (de)activated
  window.addEventListener("fullscreen", FS_onFullerScreen, false);
  // and when a page is loaded in fullscreen mode
  window.addEventListener("pageshow", CheckIfFullScreen, false);
  // and when the find toolbar is shown
  window.addEventListener("find-activated", ShowFindToolbar, false);

  // and we want to be notified when a document is loaded because some
  // content types (flash for instance) should not be rendered in
  // fullerscreen mode
  window.getBrowser().addProgressListener(
      FS_urlLocationChange,
      Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);


  // do we need to hide the statusbar icon ?
  var prefsvc =
    Components.classes["@mozilla.org/preferences-service;1"]
              .getService(Components.interfaces.nsIPrefBranch2);

  var showIcon = prefsvc.getBoolPref(FS_data.kSHOW_ICON_PREF);
  if (showIcon)
    FS_data.mStatusbarIcon.removeAttribute("hidden");
  else
    FS_data.mStatusbarIcon.setAttribute("hidden", "true");

  prefsvc.addObserver(FS_data.kSHOW_ICON_PREF,
                      FS_prefObserver, false);

  // SeaMonkey mods:
  FS_data.kBROWSER_CHROME_URI = FS_getPrefs() ?
                                  FS_getPrefs().getCharPref("browser.chromeURL") :
                                 "chrome://browser/content/browser.xul";
}

function fullerScreenShutdown()
{
  window.getBrowser().removeProgressListener(
      FS_urlLocationChange,
      Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);

  var prefsvc =
    Components.classes["@mozilla.org/preferences-service;1"]
              .getService(Components.interfaces.nsIPrefBranch2);
  prefsvc.removeObserver(FS_data.kSHOW_ICON_PREF,
                         FS_prefObserver, false);

  window.removeEventListener("fullscreen",     FS_onFullerScreen, false);
  window.removeEventListener("pageshow",       CheckIfFullScreen, false);
  window.removeEventListener("find-activated", ShowFindToolbar, false);
}

function onStatusBarDisplayChange(aEvent)
{
  if (aEvent.attrName == "label" &&
      FS_data.mStatusbarDisplayText != aEvent.newValue)
  {
    var showOnLinks = FS_getPrefs() ?
        FS_getPrefs().getBoolPref(FS_data.kHYPERLINKS_PREF) :
        true;
    if (!showOnLinks)
      return;

    // the statusbar main label changed, let's store the new value
    FS_data.mStatusbarDisplayText = aEvent.newValue;

    // and show the statusbar
    FS_toggleElementsForFullerScreen("statusbar", true, true);
    // do we already have a timeout here ? Clear it if we have
    if (FS_data.mStatusbarDisplayTimeout)
      clearTimeout(FS_data.mStatusbarDisplayTimeout);
    // show the statusbar a limited amount of time
    var delay = FS_getPrefs() ?
                  FS_getPrefs().getIntPref(FS_data.kTIMEOUT_PREF) : 4000;
    FS_data.mStatusbarDisplayTimeout = setTimeout(HideStatusBar, delay);
  }
}

function HideStatusBar()
{
  // time ran out, let's do some clean up
  FS_data.mStatusbarDisplayTimeout = null;
  FS_data.mStatusbarDisplayText = "";

  // sanity check
  if (FS_data.mIsFullScreen)
  {
    // oh and of course hide the statusbar
    FS_toggleElementsForFullerScreen("statusbar", true, false);
  }
}

function FS_onFullerScreen()
{
  // the fullscreen mode status just changed and we're notified
  var isFullScreen = window.fullScreen;

  if (!isFullScreen)
  {
    // we're now in fullscreen mode
    FS_data.mIsFullScreen = true;

    CheckIfProjection();

    // is the current MIME type compatible with fullerscreen ?
    var contentType = window.getBrowser().contentDocument.contentType;
    if (MIMETypeCantCopeWithFullerScreen(contentType))
    {
      // nope, if we enter fullerscreen mode, we may be unable to leave it
      // fallback to usual fullscreen mode
      FS_data.mForceStdFullScreen = true;

      // show toolbars, statusbar and tabs
      FS_toggleElementsForFullerScreen("toolbar", true, true);
      FS_toggleElementsForFullerScreen("statusbar", true, true);
      FS_data.mTabs.removeAttribute("moz-collapsed");
      FS_data.mIsFullerScreen = true;
    }
    else
    {
      // Luke, do you feel the power of the Force ?

      FS_data.mForceStdFullScreen = false;

      // yeah babe, groove...
      FS_data.mIsFullerScreen = false;

      // hide all what remained visible in usual fullscreen
      FS_toggleElementsForFullerScreen("toolbar", true, false);
      FS_toggleElementsForFullerScreen("statusbar", true, false);
      FS_data.mTabs.setAttribute("moz-collapsed", "true");
    }

    // we want to be notified when the statusbar label changes
    if (FS_data.mStatusbarDisplay)
      FS_data.mStatusbarDisplay.addEventListener("DOMAttrModified",
        onStatusBarDisplayChange, false);

    // and we want to detect top and bottom screen edge collisions
    window.addEventListener("mousemove", FS_onMouseMove, false);
  }
  else
  {
    // we're leaving fullscreen mode
    if (FS_data.mIsFullScreen)
    {
      window.removeEventListener("mousemove", FS_onMouseMove, false);
    }

    FS_ResetToScreen();

    FS_data.mIsFullScreen = false;
    FS_data.mIsFullerScreen = false;
    FS_data.mForceStdFullScreen = false;

    if (FS_data.mStatusbarDisplayTimeout)
      clearTimeout(FS_data.mStatusbarDisplayTimeout);
    FS_data.mStatusbarDisplayTimeout = null;

    if (FS_data.mStatusbarDisplay)
      FS_data.mStatusbarDisplay.removeEventListener("DOMAttrModified",
        onStatusBarDisplayChange, false);

    FS_toggleElementsForFullerScreen("statusbar", false);
    FS_toggleElementsForFullerScreen("toolbar", false);
    FS_data.mTabs.removeAttribute("moz-collapsed");
  }
}

function FS_onMouseMove(e)
{
  // we don't care about edge collision if we're in usual
  // fullscreen mode and not fullerscreen mode
  if (FS_data.mForceStdFullScreen)
    return;

  // what's the mouse pointer's vertical position, the target element
  // and the URL of the target element's document ?
  var tempY = e.screenY;
  var elt = e.target;
  var ode = elt.ownerDocument.documentURI;

  // are we already in FullerScreen mode ?
  if (!FS_data.mIsFullerScreen)
  {
    // yes we are, get the threshold for edge collision detection
    FS_data.kThreshold = FS_getPrefs() ?
      FS_getPrefs().getIntPref(FS_data.kTHRESHOLD_PREF) : 10;

    // do we have a top or bottom edge collision ?
    if (tempY < FS_data.kThreshold || tempY > screen.height - FS_data.kThreshold)
    {
      // yes we do, let's show the toolbars and statusbar
      FS_toggleElementsForFullerScreen("toolbar", true, true);
      FS_toggleElementsForFullerScreen("statusbar", true, true);
      FS_data.mTabs.removeAttribute("moz-collapsed");
      FS_data.mIsFullerScreen = true;
    }
  }
  else
  {
    if (ode != FS_data.kBROWSER_CHROME_URI)
    {
      // the mouse pointer hovers over content now, let's hide
      // the bars
      FS_data.mIsFullerScreen = false;
      FS_toggleElementsForFullerScreen("toolbar", true, false);
      FS_toggleElementsForFullerScreen("statusbar", true, false);
      FS_data.mTabs.setAttribute("moz-collapsed", "true");
    }
  }
}

function FS_toggleElementsForFullerScreen(aTag, aForced, aValue)
{
  // are we in usual full screen, forced or not ?
  var isFullScreen = aForced ? aValue : window.fullScreen;
  // get all the elements of the given tag aTag
  var els = document.getElementsByTagNameNS(FS_data.kXUL_NS, aTag);

  var i;
  for (i = 0; i < els.length; ++i)
  {
    // if this is a toolbar supposed to be shown in full screen mode
    // or if it's not a toolbar
    if (els[i].getAttribute("fullscreentoolbar") == "true" ||
        aTag != "toolbar")
    {
      if (isFullScreen)
        els[i].removeAttribute("moz-collapsed");
      else
        els[i].setAttribute("moz-collapsed", "true");
    }
  }
}

function MIMETypeCantCopeWithFullerScreen(aType)
{
  // get the list of MIME types incompatible with fuller screen
  // because they capture mouse and/or key events
  var badMimeTypesString = FS_getPrefs() ?
                             FS_getPrefs().getCharPref(FS_data.kBAD_MIMETYPES_PREF) :
                             "application/x-shockwave-flash,application/pdf";

  // is aType one of them ?
  var badMimeTypes = badMimeTypesString.split(",");
  var l = badMimeTypes.length;
  for (var i = 0; i < l; i++)
    if (badMimeTypes[i] == aType)
      return true;

  return false;
}

function FS_getPrefs()
{
  if (FS_data.mPrefs)
    return FS_data.mPrefs;

  try {
    FS_data.mPrefs = Components.classes["@mozilla.org/preferences-service;1"]
                               .getService(Components.interfaces.nsIPrefBranch);
  } catch (e) {
    FS_data.mPrefs = null;
  }

  return FS_data.mPrefs;
}

function FS_toggleVScrollbarStylesheet(aXULElt)
{
 var b = window.getBrowser();
 var contentType = b.contentDocument.contentType;
 if (contentType != "text/html" &&
     contentType != "application/xhtml+xml")
   return;
 var elt = b.contentDocument.documentElement;
 if (aXULElt.getAttribute("checked") == "true")
   elt.style.overflowY = "hidden";
 else
   elt.style.overflowY = "visible";
}

function FS_setVscrollbarVisibility()
{
 var b = window.getBrowser();
 var elt = b.contentDocument.documentElement;
 if (elt.style.overflowY == "hidden")
   document.getElementById("FShideVScrollbar").setAttribute("checked", "true");
 else
   document.getElementById("FShideVScrollbar").removeAttribute("checked");
}

function FS_Prefs()
{
  window.openDialog("chrome://fullerscreen/content/fs_prefs.xul", "_blank",
                    "chrome,dialog=no,close,titlebar,modal=yes,resizable=no");
}

function FS_ToggleFullScreen(aEvent)
{
  if (aEvent.button == 0 &&
      (aEvent.target.id == "FSicon" ||
       aEvent.target.id == "full-screen-statusbar-item"))
  {
    BrowserFullScreen();
  }
}

function FS_About()
{
  var tb = document.getElementById("content");
  tb.selectedTab = tb.addTab("chrome://fullerscreen/content/aboutDialog.xhtml");
}

