/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FullerScreen.
 *
 * The Initial Developer of the Original Code is
 * Disruptive Innovations SARL
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 *
 * ***** END LICENSE BLOCK ***** */
 
var FS_slideManager = {

  mSelectedSlideIndex: 0,
  mCurrentSlideIndex: 0,

  mThumbnails: null,

  kNONE: 0,
  kSELECTED: 1,
  kCURRENT: 2,

  fillSlidesMenu: function (aWindow)
  {
    // get the projection data for the current browser window
    var win = aWindow ? aWindow : GetCurrentWindow();
    var projdata = win.FS_projection_data;
    if (!projdata)
      return;

    var holder = null;
    var browser = window.getBrowser();
    var doc = browser.contentDocument;

    if (!projdata.slideManager)
    {
      // the slide manager is empty at this time
      projdata.slideManager = doc.createElement("div");

      // we rely on inline styles because we have no scoped stylesheet
      // and don't want to reduce the ID or class namespace
      projdata.slideManager.setAttribute("style",
       "visibility: hidden; \
        position: absolute; \
        top: 0px; \
        bottom: 0px; \
        left: 0px; \
        right: 0px; \
        background-image: url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH1wUQDBUnwLP0wwAAAB10RVh0Q29tbWVudABDcmVhdGVkIHdpdGggVGhlIEdJTVDvZCVuAAAAIklEQVR42mNkYGDYz0BFwMRAZTBq4KiBowaOGjhq4FAxEAA63wDnccNZcwAAAABJRU5ErkJggg==')");

      doc.documentElement.lastChild.appendChild(projdata.slideManager);

      projdata.slideManager.addEventListener("click", this.selectThumbnail, true);

      var div = doc.createElement("div");
      div.setAttribute("style",
                       "position: absolute; \
                       top: 50%; \
                       margin-top: -20px; \
                       width: 100%; \
                       overflow: auto; \
                       white-space: nowrap; \
                       font-size: x-small; \
                       color: white; \
                       font-family: sans-serif");
      projdata.slideManager.appendChild(div);

      // the main element in the slide manager is a 1-row table
      var table = doc.createElement("table");
      var tbody = doc.createElement("tbody");
      holder    = doc.createElement("tr");
      tbody.appendChild(holder);
      table.appendChild(tbody);
      div.appendChild(table);
    }
    else
    {
      // we already have a slide manager, early way out
      return;
    }

    // what's the preferred value for thumbnails' height ?
    var thumbnailHeight = FS_getPrefs() ?
                    FS_getPrefs().getIntPref(FS_data.kTHUMBNAIL_HEIGHT_PREF) : 120;

    // compute the size of canvas according to current screen width/height
    var thumbnailRatio = thumbnailHeight / screen.height;
    var thumbnailWidth = Math.floor(screen.width * thumbnailRatio);

    // browse all slides
    var slides = projdata.slides;
    if (slides && slides.length)
    {
      for (var i = 0; i < slides.length ; i++)
      {
        var slide = slides[i];
        // show each slide
        this.showFullSlide(i, win);
        // get the title of the slide if any
        var titles = getElementsBySelector(slide,
                                           projdata.titleSelector, false);
        var title = "";
        if (titles && titles.length)
          title = (i+1) + ". " + GetTextContent(titles[0]);
        else
          title = i;

        var td = doc.createElement("td");
        td.setAttribute("width", thumbnailWidth);
        td.setAttribute("align", "center");

        // create a canvas for the thumbnail
        var thumbnail = doc.createElement("canvas");
        thumbnail.setAttribute("index", i);
        thumbnail.addEventListener("click", this.selectThumbnail, true);
        thumbnail.setAttribute("style",
                               "margin-left: 2em; margin-bottom: 1em; cursor: pointer"
                              );
        thumbnail.setAttribute("width", thumbnailWidth);
        thumbnail.setAttribute("height", thumbnailHeight);
        // generate the thumbnail into the canvas
        gCanvasCopyObject(window.getBrowser(), thumbnail);
        td.appendChild(thumbnail);
        td.appendChild(doc.createElement("br"));

        var t = doc.createTextNode(title);
        td.appendChild(t);
        holder.appendChild(td);
      }
    }
    // go back to first slide
    ShowSlide(0, win, +1);
  },

  showFullSlide: function (aSlideIndex, aWin)
  {
    // get the projection data for the current browser
    var win = aWin ? aWin : GetCurrentWindow();
    var projdata = win.FS_projection_data;

    if (!projdata)
      return;

    // get the slide and show that one only
    var slides = projdata.slides;
    if (slides && slides.length)
    {
      for (var i = 0; i < slides.length ; i++)
      {
        if (i == aSlideIndex)
          slides[i].style.display = "block";
        else
          slides[i].style.display = "none";
      }
    }
  },

  showSlideManager: function ()
  {
    // get the projection data for the current browser
    var win = GetCurrentWindow();
    var projdata = win.FS_projection_data;
    if (!projdata)
      return;

    // get the slide manager
    var slidemanager = projdata.slideManager;
    slidemanager.style.visibility = "visible";

    // outline the tumbnail of the current slide
    this.mThumbnails = slidemanager.getElementsByTagName("canvas");
    for (var i = 0 ; i < this.mThumbnails.length; i++)
    {
      var t = this.mThumbnails[i];
      if (i == projdata.currentSlide)
      {
        this.highlightElement(t, this.kSELECTED);
        Components.classes["@mozilla.org/inspector/flasher;1"]
                  .getService(Components.interfaces.inIFlasher)
                  .scrollElementIntoView(t);
      }
      else
        this.highlightElement(t, this.kNONE);
    }
    this.mCurrentSlideIndex = projdata.currentSlide;
    this.mSelectedSlideIndex = projdata.currentSlide;

    // add a few event listeners for click and key
    var doc = slidemanager.ownerDocument;
    doc.documentElement.removeEventListener("keyup", KeyUp, false);
    doc.documentElement.addEventListener("keyup", this.slideManagerKeyUp, false);
  },

  highlightElement: function(aElt, aType)
  {
    if (!aElt)
      return;
    var hStyle = "";
    switch (aType)
    {
      case this.kSELECTED: hStyle = "thick red";     break;
      case this.kCURRENT:  hStyle = "thick #5A39FD"; break;
      default:             hStyle = "thin black";   break;
    }
    aElt.setAttribute("style",
                      aElt.getAttribute("style") + 
                      ";-moz-outline: " + hStyle + " solid");
  },

  hideSlideManager: function ()
  {
    // get the projection data for the current browser
    var win = GetCurrentWindow();
    var projdata = win.FS_projection_data;
    if (!projdata)
      return;

    // hide the slide manager
    var slidemanager = projdata.slideManager;
    slidemanager.style.visibility = "hidden";

    // remove the event listeners and reinstall the previous ones
    var doc = slidemanager.ownerDocument;
    doc.documentElement.removeEventListener("keyup", this.slideManagerKeyUp, false);
    doc.documentElement.addEventListener("keyup", KeyUp, false);
  },

  moveCurrentSlide: function(aIncrement)
  {
    var slide = FS_slideManager.mThumbnails[FS_slideManager.mCurrentSlideIndex];
    FS_slideManager.highlightElement(slide,
         FS_slideManager.mCurrentSlideIndex == FS_slideManager.mSelectedSlideIndex ?
            FS_slideManager.kSELECTED : FS_slideManager.kNONE);

    var newSlideIndex = FS_slideManager.mCurrentSlideIndex + aIncrement;
    if (newSlideIndex < 0)
      newSlideIndex = 0;
    else if (newSlideIndex >= FS_slideManager.mThumbnails.length)
      newSlideIndex = FS_slideManager.mThumbnails.length - 1;
    FS_slideManager.mCurrentSlideIndex = newSlideIndex;
    slide = FS_slideManager.mThumbnails[newSlideIndex];
    FS_slideManager.highlightElement(slide, FS_slideManager.kCURRENT);
    Components.classes["@mozilla.org/inspector/flasher;1"]
              .getService(Components.interfaces.inIFlasher)
              .scrollElementIntoView(slide);
  },

  slideManagerKeyUp: function (aEvent) {
    switch (aEvent.which) {
      case 122: // F11
        if (aEvent.shiftKey)
          FS_slideManager.toggleSlideManager();
        break;
      case 27:  // ESC
        FS_slideManager.toggleSlideManager();
        break;
      case 32: // spacebar
      case 13: // CR
        FS_slideManager.hideSlideManager();
        ShowSlide(FS_slideManager.mCurrentSlideIndex, null, +1);
        break;
      case 34: // page down
      case 39: // rightkey
      case 40: // downkey
        FS_slideManager.moveCurrentSlide(+1);
        break;
      case 33: // page up
      case 37: // leftkey
      case 38: // upkey
        FS_slideManager.moveCurrentSlide(-1);
        break;
      case 36: // home
        FS_slideManager.goToSlide(0);
        break;
      case 35: // end
        FS_slideManager.goToSlide(-1);
        break;
      default:
        break;
    }
    return false;
  },

  goToSlide: function(aIndex)
  {
    var slide = FS_slideManager.mThumbnails[FS_slideManager.mCurrentSlideIndex];
    FS_slideManager.highlightElement(slide,
         FS_slideManager.mCurrentSlideIndex == FS_slideManager.mSelectedSlideIndex ?
            FS_slideManager.kSELECTED : FS_slideManager.kNONE);

    var newSlideIndex = aIndex;
    if (newSlideIndex < 0)
      newSlideIndex = FS_slideManager.mThumbnails.length - 1;

    FS_slideManager.mCurrentSlideIndex = newSlideIndex;
    slide = FS_slideManager.mThumbnails[newSlideIndex];
    FS_slideManager.highlightElement(slide, FS_slideManager.kCURRENT);
    Components.classes["@mozilla.org/inspector/flasher;1"]
              .getService(Components.interfaces.inIFlasher)
              .scrollElementIntoView(slide);
  },

  selectThumbnail: function (aEvent)
  {
    // did the click occur on a thumbnail ?
    if (aEvent.target.nodeName.toLowerCase() == "canvas")
    {
      var index = parseInt(aEvent.target.getAttribute("index"));
      // hide the slide manager and show the slide
      FS_slideManager.hideSlideManager();
      ShowSlide(index, null, +1);
    }
    // make sure the event is not captured by somebody else
    aEvent.stopPropagation();
    return false;
  },

  toggleSlideManager: function ()
  {
    var win = GetCurrentWindow();
    var projdata = win.FS_projection_data;
    if (!projdata)
      return;
    var slidemanager = projdata.slideManager;
    if (slidemanager.style.visibility == "hidden")
      this.showSlideManager();
    else
      this.hideSlideManager();
  }
};
